//*****************************************************************************
//                              CmdGnuCapDC.hpp                               *
//                             -----------------                              *
//  Description : This class contains values associated with the GNU-Cap DC   *
//                command. It inherits from the class CmdBase.                *
//  Started     : 29/01/2008                                                  *
//  Last Update : 21/02/2008                                                  *
//  Copyright   : (C) 2008 by M.S.Waters                                      *
//  Email       : M.Waters@bom.gov.au                                         *
//*****************************************************************************

//*****************************************************************************
//                                                                            *
//    This program is free software; you can redistribute it and/or modify    *
//    it under the terms of the GNU General Public License as published by    *
//    the Free Software Foundation; either version 2 of the License, or       *
//    (at your option) any later version.                                     *
//                                                                            *
//*****************************************************************************

#ifndef CMDGNUCAPDC_HPP
#define CMDGNUCAPDC_HPP

// System Includes

using  namespace  std;

#include <iostream>
#include <climits>

// wxWidgets Includes

#include <wx/wx.h>
#include <wx/tokenzr.h>

// Application Includes

#include "TypeDefs.hpp"
#include "base/CmdBase.hpp"
#include "utility/ConvertType.hpp"

// Local Macro and Constant Declarations

//*****************************************************************************

struct CmdGnuCapDC : public CmdBase
{
  public :

    float       m_fStart;    // Sweep start value
    float       m_fStop;     // Sweep stop  value
    float       m_fStep;     // Sweep step  value
    wxString    m_osSource;  // Sweep source
    eScaleType  m_eScale;    // Step scale
    float       m_fTempC;    // Analysis temperature in Celcius

              CmdGnuCapDC( void );
    virtual  ~CmdGnuCapDC( );

    bool  bClear( void );

    virtual  bool  bParse ( void );
    virtual  bool  bFormat( void );

    wxString & operator = ( const wxString & ros )
                { *((wxString *) this) = ros; return( *((wxString *) this) ); }

    void  Print( const char * psPrefix="" );
};

//*****************************************************************************

#endif // CMDGNUCAPDC_HPP
